"""
Transfer style of an animation with the method Holden/Gatys method:
compute descriptor with the autoencoder => Gram Matrix of the code

Assumption: 2 animations has similar style if Gram Matrix of code are similar => run an optimisation on animation,
then a constraints solver for length, feet sliding ?
"""

import os
import sys
import argparse
import math
from io import StringIO
import numpy as np

import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim

import torchvision
from torchvision import datasets, transforms
from torch.autograd import Variable
from torchvision.datasets import ImageFolder
from torch.utils.data import Dataset, DataLoader
from torchvision.transforms import ToTensor
from torch.utils.data.sampler import SubsetRandomSampler
from torch.utils.data.sampler import SubsetRandomSampler
from torchsummary import summary

import pydeepan as pdp
from pydeepan import log
from pydeepan.chara.AOSkeletonConstraints import AnimationOptimizer
from pydeepan.latent.AutoEncoder import AutoEncoder
from pydeepan.latent.AutoEncoder import AutoEncoderNet
#from pydeepan.dataset.HPADataset import HPADataset
from pydeepan.dataset.HPADataset import HPANormalize
from pydeepan.chara.HPAAnimation import HPAAnimation


#from InverseKinematics import BasicJacobianIK, JacobianInverseKinematics



class AOStyleTransfer(AnimationOptimizer):
    """
        Transfer style of an animation with optimization: Gatys's like transfer

    """

    def __init__(self, device=torch.device("cpu")):
        super().__init__(device)
        self.ae_net = self.__autoenc_load( "cmu", device )
        self.hpanorm = HPANormalize( "cmu" )
        print( str(self.ae_net) )
        print( str(self.hpanorm) )


    def __autoenc_load(self, ae_name, device):
        ae_name = AutoEncoder.filenames(ae_name, best=True)
        print("Reco::autoenc_load: load autoencoder="+ae_name)
        ae_net = AutoEncoderNet( device, False)
        ae_net_brut = torch.load(ae_name, map_location=device)
        ae_net.load_state_dict( ae_net_brut.state_dict() )
        ae_net.eval()              # set net mode to 'eval' (not traning)
        return ae_net


    def encode_decode(self, anim:HPAAnimation):
        """
            encode + decode
        """
        code, ind = self.encode( anim )
        res = self.decode( code, ind )
        return res


    def decode(self, code:torch.tensor, ind:torch.tensor)->HPAAnimation:
        """
            pass anim through the autoencoder: just decoder and postprocess
        """
        res = self.ae_net.decode( code, ind )
        res = self.hpanorm.postprocess( res.detach().cpu().numpy() )           # rescale animation to original values
        return res


    def encode(self, anim:HPAAnimation)->(torch.tensor, torch.tensor):
        """
            pass anim through the autoencoder: preprocess with the HPANormalize, encode and return the code + indice for max pooling
        """
        a = self.hpanorm.preprocess( anim )            # normalize animation
        a = torch.from_numpy(a).float().to( self.ae_net.device )
        code, ind = self.ae_net.encode( a, True )
        return code, ind


    def gram(self, tensor:torch.tensor):
        """ Calculate the Gram Matrix of a given tensor 
            Gram Matrix: https://en.wikipedia.org/wiki/Gramian_matrix
        """
        # TODO
        #return gram


    def loss(self, anim:torch.tensor, content:torch.tensor, style:torch.tensor):
        #TODO
        #a = self.ae_net.encode( b )            # exemple de passage dans encoed ou decode
        loss_term = 0
        return loss_term


    def optimize(self, content:HPAAnimation, style:HPAAnimation, n=10000, print_loss=False)->HPAAnimation:
        """
         transfer style from source to destination
        """
        content_prepro = self.hpanorm.preprocess( [content] )              # preprocessing: it is done by the dataset (it normalizes the animations)
        content_prepro = torch.tensor( content_prepro ).to( self.device).float()

        style_prepro = self.hpanorm.preprocess( [style] )                  # preprocessing: it is done by the dataset (it normalizes the animations)
        style_prepro = torch.tensor( style_prepro ).to( self.device).float()

        result_tensor = torch.rand(size=content_prepro.size(), requires_grad=True, dtype=torch.float, device=self.device)     # create the resulting tensor


        if self.optimizer == None:
            self.optimizer = optim.SGD( [result_tensor] , lr=0.1, momentum=0.6)

        #TODO: iteration / optimization



        # result = result_tensor.to("cpu").clone().detach().numpy()
        # result = self.hpanorm.postprocess( result )           # rescale animation to original values
        # result = HPAAnimation.cast( result )
        return result






if __name__ == "__main__":
    print("END")
