"""
## Demo of expression recobatch gnition from body movement
##
##
"""

import math
import sys
import os
import numpy as np
from PIL import Image

import panda3d
from direct.gui.OnscreenText import OnscreenText
from direct.gui.OnscreenText import TextNode
from panda3d.core import NodePath
from direct.gui.OnscreenImage import OnscreenImage
from panda3d.core import PNMImage
from panda3d.core import Filename
from panda3d.core import Texture
from panda3d.core import CPTAUchar, PTAUchar, PTAStdfloat

import pydeepan
from pydeepan import log
from pydeepan.torch_import import *
from pydeepan import dataset as AnimDataset
from pydeepan.dataset.HPADataset import *
from pydeepan.viewer.PandaAnimViewer import PandaAnimViewer
from pydeepan.viewer.PandaSkeleton import HPAPandaSkeleton

from pydeepan.dataset.HPADataset import HPADataset
from pydeepan.demo.HPADatasetDemo import HPADatasetDemoViewer
from pydeepan.latent.AutoEncoder import AutoEncoderNet

from pydeepan.chara.HPAAnimation import HPAAnimation
from pydeepan.chara.AOStyleTransfer import AOStyleTransfer
from pydeepan.chara.AOSkeletonConstraints import AOSkeletonConstraints




class StyleTransferDemoViewer(HPADatasetDemoViewer):
    """
        A viewer to illustrate the dataset usage in pytorch + display with panda/PandaAnimViewer

        Arguments:
            - device:
            - style_transfer(StyleTransfer): 
    """

    def __init__(self):
        HPADatasetDemoViewer.__init__(self, 
                                            dataset_names=[ "styletransfer", "mixing", "emilya" ], 
                                            nb_anim_batch=2*2,              # 4 animations displayed, src, dst and transformed_src, dst_transformed
                                            nb_anim_from_dataset=2,         # 2 animations from the dataset. The 2 others will be the transformed animations
                                            win_title="StyleTransferDemoViewer")

        self.device = torch.device("cuda")
        self.style_transfer = AOStyleTransfer( device=self.device )
        self.nextBatch()
        self.length_optim = AOSkeletonConstraints(   anim=HPAAnimation.cast(self.anims[0]), device=self.device )

        # key
        self.addKey("e", "Animation: one pass of autoencoder")
        self.addKey("l", "Animation: one pass of limb length contraints")
        self.addKey("t", "Animation: style transfer: 0->1 in 2 and 1->0 in 3")
        self.addKey("r", "Animation: break an animation")
        self.addKey("i", "Animation: interpolation")


    def nextBatch(self):
        """
           generate a new batch of animations.
        """
        super().nextBatch()         # it will extract nb_anim_batch/2=nb_anim_from_dataset from the dataset
        for i in range(self.batch_from_dataset_size):
            self.anims = np.append( self.anims, [self.anims[i]], axis=0 )           # duplicate animations: half is original anim/half is modifier animation (autoencoded)
            self.labels = np.append( self.labels, [self.labels[i]], axis=0 )        # dupliacte labels: half is original anim/half is modifier animation (autoencoded)

            i_new = self.batch_from_dataset_size + i
            ske = self.pandaskeletons[ i_new ]
            ske.setColorStyle(2)
            self.setAnimationPosXY( i_new, self.labels[i_new],  self.labels_count[ self.labels[i_new] ])
            self.labels_count[ self.labels[i_new] ] += 1
            txtadd = "m"                                        # 'm' for modified
            ske.setId( self.pandaskeletons[ i ].id, txtadd )
        self.txtDataset.text = self.dataset_txt()+"Half animations are transformed"
        self.setPosesOfAllSkeletons()



    def keyHandler(self,key):
        if key=="e":
            for i in range(self.batch_from_dataset_size):
                i_new = self.batch_from_dataset_size + i
                r = self.style_transfer.encode_decode( np.array( [self.anims[i_new],] ) )
                self.anims[i_new] = r[0]
            self.setPosesOfAllSkeletons()
            print("Encode/Decode: done")
        elif key=="l":
            for i in range(self.batch_from_dataset_size):
                i_new = self.batch_from_dataset_size + i
                r = self.anims[i_new]
                r = self.length_optim.optimize( HPAAnimation.cast(r), n=1000, print_loss=True )
                self.anims[i_new] = r
            self.setPosesOfAllSkeletons()
            print("Opti length constraints: done")
        elif key=="t":
            src = self.anims[0]
            dst = self.anims[1]
            r0 = self.style_transfer.optimize( src, dst, print_loss=True)
            #r1 = self.style_transfer.optimize( dst, src, self.dataset)
            r1 = r0
            self.anims[2] = r0
            self.anims[3] = r1
            self.setPosesOfAllSkeletons()
            print("style transfer: 2=style from 0 to 1 and 3=from 1 to 0")
        elif key=="r":
            # TODO
            # ...
            self.setPosesOfAllSkeletons()
            print("random animations")
        elif key=="i":
            # TODO
            # ...
            self.setPosesOfAllSkeletons()
            print("interpolation ")
            
        else:
            super().keyHandler(key)


    def animate(self):
        super().animate()




if __name__ == '__main__' and sys.flags.interactive == 0:
    app = StyleTransferDemoViewer()
    app.run()
