
import pydeepan
from pydeepan import log, tools
from pydeepan.torch_import import *
from pydeepan import dataset
from pydeepan.dataset.HPADataset import *
from pydeepan.latent.IdentityEncoder import IdentityEncoder


class GramEncoder(IdentityEncoder):
    """
    ################# GRAM Auto Encoder######################################
    # input_shape with out the batch size
    """
    def __init__(self, input_shape=[73,240], device=torch.device("cpu") ):
        IdentityEncoder.__init__(self, input_shape, device)


    def encode(self, tensor):
        """ Calculate the Gram Matrix of a given tensor 
            Gram Matrix: https://en.wikipedia.org/wiki/Gramian_matrix
        """
        # get the batch_size, depth, height, and width of the Tensor
        if tensor.dim()==2:
            h, w = tensor.size()
            gram = torch.empty( [h,h] )
            gram = torch.mm(tensor, tensor.t())
            return gram
        if tensor.dim()==3:
            b, h, w = tensor.size()
            # print("Gram size input=",d,h,w)
            # reshape so we're multiplying the features for each channel
            #tensor = tensor.view(d, h * w)
            #tensor = tensor.view(h, w)
            # h, w = tensor.size()
            # print("Gram: tensor x tensorT =",h,w)
            # calculate the gram matrix
            gram = torch.empty( [b,h,h] )
            for i in range(b):
                gram[i] = torch.mm(tensor[i], tensor[i].t())
            # h, w = gram.size()
            # print("Gram size output=",h,w)
            return gram





if __name__ == "__main__":
    print("Encoder...")

    gram = GramEncoder( [73,240], torch.device("cpu"))
    print(gram)

    hpa_transform = transforms.Compose([
                    HPATransformClipWithoutLabel(),
                    gram
                ])

    anims_npz_filename = dataset.name2filename('styletransfer')
    log_file = pydeepan.dir_pydeepan + "data/GramEncoder.log"
    log = log.create_logger(log_file)    
    hpadataset = HPADataset( anims_npz_filename, hpa_transform )
    loader = DataLoader( hpadataset, batch_size=4, shuffle=True, num_workers=0)
    dataiter = iter(loader)
    clips = dataiter.next()
    print("animation batch: type="+ str(type(clips)) +" size="+str(len(clips)))

    clips_enc = id(clips)
    print( str(clips_enc))

