import pydeepan
from pydeepan import log, tools
from pydeepan.torch_import import *
from pydeepan import dataset
from pydeepan.dataset.HPADataset import *


class IdentityEncoder():
    def __init__(self, input_shape=[73,240], device=torch.device("cpu") ):
        self.device = device
        self._input_shape = input_shape
        self._code_shape = self.__compute_code_shape()

    @property
    def code_shape(self):
        return self._code_shape
    
    @property
    def input_shape(self):
        return self._input_shape

    def __compute_code_shape(self):
        input = torch.rand( [4] + list(self.input_shape))      # batch of 4 x 73 channels x 240 frame
        code = self.encode(input)
        return code.size()[1:]      # suppress batch dim

    def __str__(self):
        return "### "+type(self).__name__+": input_shape="+str(self.input_shape)+ "  code_shape="+str(self.code_shape) + "  device="+str(self.device)

    # encode th
    def encode(self, x):
        print("IdentityEncoder::encode: "+str(type(x))+ "  "+str(x.shape))
        return x

    def __call__(self, x):
        y = x
        if isinstance(x,list):
            y = x[0]
            y = y.to(self.device)
            return [self.encode(y), x[1] ]
        elif isinstance(x,tuple):
            y = x[0]
            y = y.to(self.device)
            return ( self.encode(y), x[1] )
        else:
            y.to(self.device)
            return self.encode(y)






# ################# GRAM ######################################
# class PipeEncoder(IdentityEncoder):
#     def __init__(self, enc1, enc2):
#         self.enc1 = enc1
#         self.enc2 = enc2

#     def print(self):
#         print("### PipeEncode: enc1|enc2.   enc1:")
#         self.enc1.print()
#         print("### PipeEncode: enc1|enc2.   enc2:")
#         self.enc2.print()

#     def encode(self,x):
#         xx = self.enc1.encode(x)
#         xxx = self.enc2.encode(xx)
#         return xxx




if __name__ == "__main__":
    print("Encoder...")

    id = IdentityEncoder( [73,240], torch.device("cpu"))
    print(id)

    hpa_transform = transforms.Compose([
                    HPATransformClipWithoutLabel(),
                    id
                ])

    anims_npz_filename = dataset.name2filename('styletransfer')
    log_file = pydeepan.dir_pydeepan + "data/IdentityEncoder.log"
    log = log.create_logger(log_file)    
    hpadataset = HPADataset( anims_npz_filename, hpa_transform )
    loader = DataLoader( hpadataset, batch_size=4, shuffle=True, num_workers=0)
    dataiter = iter(loader)
    clips = dataiter.next()
    print("animation batch: type="+ str(type(clips)) +" size="+str(len(clips)))

    clips_enc = id(clips)
    print(clips_enc.shape)

