'''
VGG encode
'''

import os
import sys
import math
import numpy as np

import pydeepan as pdp
from pydeepan import dataset as ds
from pydeepan.viewer.PlotViewer import PlotDraw
from pydeepan import log
from pydeepan.latent.IdentityEncode import IdentityEncoder

import torch
import torch.optim as optim
from torchvision import transforms, models

from torchsummary import summary



##########################" VGG "#########################################################""
class VGGEncoder(IdentityEncoder):
    def __init__(self, input_shape=[73,240], device=torch.device("cpu")):
        self.device = device
        # get the "features" portion of VGG19 (we will not need the "classifier" portion)
        self.vgg = models.vgg19(pretrained=True).features
        self.vgg.to( self.device )
        features = list(self.vgg)[:23]
        for i,layer in enumerate(features):
            print(i,"   ",layer)
        IdentityEncoder.__init__(self, input_shape)

    def print(self):
        print("### VGGEncoder:")
        self.vgg.to(self.device)
        print("code shape=" + str(self.code_shape))
        summary(self.vgg, self.input_size, device=self.device.type)

    ### Run an image forward through a model and get the features for a set of layers. 
    def __getfeatures__(self, image):
        ## Need the layers for the content and style representations of an image
        layers = None
        if layers is None:
            layers = {'0': 'conv0',
                    '5': 'conv5', 
                    '10': 'conv10', 
                    '19': 'conv19',
                    '21': 'conv21',  ## content representation
                    '28': 'conv28'}

        features = {}
        x = image
        # model._modules is a dictionary holding each module in the model
        for name, layer in self.vgg._modules.items():
            x = layer(x)
            if name in layers:
                features[layers[name]] = x            
        return features

    def encode(self, x):
        s = x.shape
        xx = x.view( s[0], s[1], s[2], 1)
        return self.__getfeatures__(xx)







if __name__ == "__main__":
    print("Encoder...")

    id = IdentityEncoder()
    id.print()

    #vgge = VGGEncoder()
    #vgge.print()


    anims_npz_filename = animation_ds.name2filename('styletransfer')
    log_file = pydeepan.dir_pydeepan + "data/VGGEncoder.log"
    log = log.create_logger(log_file)
    hpadataset = HPADataset( anims_npz_filename, log )

    dataiter = iter(loader)
    clips = dataiter.next()

