
import logging
from logging.handlers import RotatingFileHandler
from contextlib import redirect_stdout

def redirect(log):
    redirect_stdout(log)

class LoggerWriter:
    def __init__(self, logger):
        self.logger = logger

    def write(self, message):
        if message != '\n':
            self.logger.info(message)


def create_logger(filename, do_redirect=True):
    # création de l'objet logger qui va nous servir à écrire dans les logs
    log = logging.getLogger()
    # on met le niveau du logger à DEBUG, comme ça il écrit tout
    log.setLevel(logging.DEBUG)
 
    # création d'un formateur qui va ajouter le temps, le niveau
    # de chaque message quand on écrira un message dans le log
    formatter = logging.Formatter('%(asctime)s :: %(levelname)s :: %(message)s')
    # création d'un handler qui va rediriger une écriture du log vers
    # un fichier en mode 'append', avec 1 backup et une taille max de 100Mo
    file_handler = RotatingFileHandler(filename, 'a', 100000000, 1)
    # on lui met le niveau sur DEBUG, on lui dit qu'il doit utiliser le formateur
    # créé précédement et on ajoute ce handler au logger
    file_handler.setLevel(logging.DEBUG)
    file_handler.setFormatter(formatter)
    file_handler.doRollover()
    #file_handler.terminator = ""
    log.addHandler(file_handler)
 
    # création d'un second handler qui va rediriger chaque écriture de log
    # sur la console
    stream_handler = logging.StreamHandler()
    #stream_handler.terminator = ""
    stream_handler.setLevel(logging.DEBUG)
    log.addHandler(stream_handler)
 
    if do_redirect:
        return redirect_stdout( LoggerWriter(log) )

    return log

