"""
    Transform = Ve3 + Quaternion
"""

import sys
from typing import Type, TypeVar
import numpy as np

from glm import vec3
from glm import quat
from glm import angleAxis
from glm import inverse
from glm import radians


TTransform = TypeVar("TTransform", bound="Transform")

class Transform():
    """
        Vec3 + Quaternion

        Arguments:
            tr: vec3, translation
            ro: quaternion, rotation
    """

    def __init__(self, r=quat(), t=vec3(0,0,0) ):
        self._ro = r
        self._tr = t


    @staticmethod    
    def translate(tr:vec3) -> TTransform:
        return Transform( t=tr )


    @staticmethod    
    def rotate( angle:float, axis:vec3) -> TTransform:
        return Transform( r = angleAxis( radians(angle), axis) )


    @property
    def translation(self) -> vec3:
        return self._tr

    def setTranslation(self, tr):
        self._tr = tr


    @property
    def rotation(self) -> quat:
        return self._ro


    def set_identity(self):
        self = Transform()


    def __add__(self, o:TTransform)->TTransform:
        self._ro += o._ro
        self._tr += o._tr
        return self


    def inverse(self)->TTransform:
        #return TTransformQ(glm::inverse(Q), VecReal(0,0,0))   *   TTransformQ( -T );
        return Transform(  ro=inverse(self._ro)) *  Transform( tr=-self._tr )


    def __mul__(self, o:TTransform)->TTransform:
        """
            o (Transform or vec3)
        """
        if isinstance(o, Transform):
            return Transform(  self._ro*o._ro,  self._ro * o._tr + self._tr )
        elif isinstance(o, vec3):
            return self._ro*o
        else:
            raise TypeError("Only vec3 or Transform") 


    def __str__(self)->str:
        txt = "(ro="+str(self._ro) + "tr=" + str(self._tr) + ")"
        return txt





if __name__ == "__main__":
    t = Transform()
    tr = Transform.translate( vec3(1,2,3) )
    rx = Transform.rotate( 90.0, vec3(1,0,0))

    r0 = rx * rx * rx * rx

    print( "t=" + str(t) )
    print( "tr=" + str(tr) )
    print( "rx=" + str(rx) )
    print( "r0=" + str(r0) )
    print("sizeof(Transform)=" + str(sys.getsizeof(t)) )

