
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.animation as animation
import matplotlib.colors as colors
from matplotlib.animation import ArtistAnimation
import matplotlib.patheffects as pe
import os, sys

from pydeepan import dataset
from pydeepan.motion.Quaternions import Quaternions



def HPAPlotDraw(animationsnp, interval=33.33):         # draw an numpy array of HPA animations

    footsteps = []
    animations = []
    for ai in range(animationsnp.shape[0]):
        #anim = animations[ai].copy()[:,3:]
        anim = animationsnp[ai].copy()[:,3:]

        joints, root_x, root_z, root_r = anim[:,:-7], anim[:,-7], anim[:,-6], anim[:,-5]
        joints = joints.reshape((len(joints), -1, 3))

        rotation = Quaternions.id(1)
        offsets = []
        translation = np.array([[0,0,0]])

        for i in range(len(joints)):
            joints[i,:,:] = rotation * joints[i]
            joints[i,:,0] = joints[i,:,0] + translation[0,0]
            joints[i,:,2] = joints[i,:,2] + translation[0,2]
            rotation = Quaternions.from_angle_axis(-root_r[i], np.array([0,1,0])) * rotation
            offsets.append(rotation * np.array([0,0,1]))
            translation = translation + rotation * np.array([root_x[i], 0, root_z[i]])

        #animations[ai] = joints
        animations.append( joints )
        footsteps.append(anim[:,-4:])

    footsteps = np.array(footsteps)

    scale = 1.25*((len(animations))/2)

    fig = plt.figure(figsize=(12,8))
    ax = fig.add_subplot(111, projection='3d')
    ax.set_xlim3d(-scale*30, scale*30)
    ax.set_zlim3d( 0, scale*60)
    ax.set_ylim3d(-scale*30, scale*30)
    ax.set_xticks([], [])
    ax.set_yticks([], [])
    ax.set_zticks([], [])
    # ax.set_aspect('equal')   # alex

    acolors = list(sorted(colors.cnames.keys()))[::-1]
    lines = []

    parents = np.array([-1,0,1,2,3,0,5,6,7,0,9,10,11,11,13,14,15,11,17,18,19])

    for ai, anim in enumerate(animations):
        lines.append([plt.plot([0,0], [0,0], [0,0], color=acolors[ai],
            lw=2, path_effects=[pe.Stroke(linewidth=3, foreground='black'), pe.Normal()])[0] for _ in range(anim.shape[1])])

    def animate(i):
        changed = []
        for ai in range(len(animations)):
            offset = 25*(ai-((len(animations))/2))
            for j in range(len(parents)):
                if parents[j] != -1:
                    lines[ai][j].set_data(
                        [ animations[ai][i,j,0]+offset, animations[ai][i,parents[j],0]+offset],
                        [-animations[ai][i,j,2],       -animations[ai][i,parents[j],2]])
                    lines[ai][j].set_3d_properties(
                        [ animations[ai][i,j,1],        animations[ai][i,parents[j],1]])
            changed += lines

        return changed

    # The fix
    for spine in ax.spines.values():
        spine.set_visible(False)

    plt.tight_layout()

    ani = animation.FuncAnimation(fig,
        animate, np.arange(len(animations[0])), interval=interval)

    plt.show()



def animation_plot_orig(animations, interval=33.33):         # draw a list of Animations

    footsteps = []

    for ai in range(len(animations)):
        #anim = animations[ai].copy()[:,3:]
        anim = animations[ai][0].copy()[:,3:]

        joints, root_x, root_z, root_r = anim[:,:-7], anim[:,-7], anim[:,-6], anim[:,-5]
        joints = joints.reshape((len(joints), -1, 3))

        rotation = Quaternion.id(1)
        offsets = []
        translation = np.array([[0,0,0]])

        for i in range(len(joints)):
            joints[i,:,:] = rotation * joints[i]
            joints[i,:,0] = joints[i,:,0] + translation[0,0]
            joints[i,:,2] = joints[i,:,2] + translation[0,2]
            rotation = Quaternions.from_angle_axis(-root_r[i], np.array([0,1,0])) * rotation
            offsets.append(rotation * np.array([0,0,1]))
            translation = translation + rotation * np.array([root_x[i], 0, root_z[i]])

        animations[ai] = joints
        footsteps.append(anim[:,-4:])

    footsteps = np.array(footsteps)

    scale = 1.25*((len(animations))/2)

    fig = plt.figure(figsize=(12,8))
    ax = fig.add_subplot(111, projection='3d')
    ax.set_xlim3d(-scale*30, scale*30)
    ax.set_zlim3d( 0, scale*60)
    ax.set_ylim3d(-scale*30, scale*30)
    ax.set_xticks([], [])
    ax.set_yticks([], [])
    ax.set_zticks([], [])
    # ax.set_aspect('equal')   # alex

    acolors = list(sorted(colors.cnames.keys()))[::-1]
    lines = []

    parents = np.array([-1,0,1,2,3,0,5,6,7,0,9,10,11,11,13,14,15,11,17,18,19])

    for ai, anim in enumerate(animations):
        lines.append([plt.plot([0,0], [0,0], [0,0], color=acolors[ai],
            lw=2, path_effects=[pe.Stroke(linewidth=3, foreground='black'), pe.Normal()])[0] for _ in range(anim.shape[1])])

    def animate(i):
        changed = []
        for ai in range(len(animations)):
            offset = 25*(ai-((len(animations))/2))
            for j in range(len(parents)):
                if parents[j] != -1:
                    lines[ai][j].set_data(
                        [ animations[ai][i,j,0]+offset, animations[ai][i,parents[j],0]+offset],
                        [-animations[ai][i,j,2],       -animations[ai][i,parents[j],2]])
                    lines[ai][j].set_3d_properties(
                        [ animations[ai][i,j,1],        animations[ai][i,parents[j],1]])
            changed += lines

        return changed

    # The fix
    for spine in ax.spines.values():
        spine.set_visible(False)

    plt.tight_layout()

    ani = animation.FuncAnimation(fig,
        animate, np.arange(len(animations[0])), interval=interval)

    plt.show()





if __name__ == "__main__":

    #database = np.load( dataset.path_processed + 'data_cmu.npz')['clips']
    database = np.load( dataset.dir_anims_processed_npz + 'data_hdm05.npz')['clips']
    #database = np.load( dataset.path_processed + 'data_edin_locomotion.npz')['clips']

    print("database shape="+str(database.shape))

    anim1 = database[0]
    anim2 = database[5]
    anim3 = database[8]
    print("anim shape="+str(anim1.shape))

    aa = np.array( [ anim1, anim2, anim3] )
    print("animS shape="+str(aa.shape))
    #aa = [ anim1, anim2, anim3]

    HPAPlotDraw(aa)



    #for i in range(1):

    #    index0 = np.random.randint(0, len(database))
    #    index1 = np.random.randint(0, len(database))
    #    index2 = np.random.randint(0, len(database))

    #    animation_plot([
    #        database[index0:index0+1],
    #        database[index1:index1+1],
    #        database[index2:index2+1],
    #    ])
