
from math import pi, sin, cos
import sys
import os
import math
import numpy as np
import code

from direct.showbase.ShowBase import ShowBase
from direct.showbase.DirectObject import DirectObject
from panda3d.core import lookAt
from panda3d.core import GeomVertexFormat, GeomVertexData
from panda3d.core import Geom, GeomTriangles, GeomVertexWriter
from panda3d.core import Texture, GeomNode, CardMaker
from panda3d.core import PerspectiveLens
from panda3d.core import Light, PointLight, Spotlight, DirectionalLight, AmbientLight
#from panda3d.core import TextNode
from panda3d.core import LVector3, LQuaternion
from panda3d.core import Filename, LineSegs, NodePath
from direct.task import Task
from panda3d.core import CompassEffect
from panda3d.core import KeyboardButton
from direct.showbase.InputStateGlobal import inputState
#from pandac.PandaModules import Base
from panda3d.core import loadPrcFileData


# camera+mouse: see https://discourse.panda3d.org/t/camera-controls/8718/2
#
# TODO
# @todo simple UI for any variable
# @todo better camera control?

def normalized(*args):
    myVec = LVector3(*args)
    myVec.normalize()
    return myVec


class PandaViewer(ShowBase):
    """
        Base class
        Arguments:

    """

    def __init__(self, win_size="1280 768", win_title="PandaViewer"):
        print("current dir: " + os.getcwd())
        loadPrcFileData('', 'win-size '+win_size)
        loadPrcFileData('', 'window-title '+win_title)
        loadPrcFileData('', 'sync-video #f')            # disable synchro with screen
        ShowBase.__init__(self)
        base.setBackgroundColor(0.6, 0.8, 0.9, 1.0)
        #self.setTitle(win_title)
        self.interactive_debug = False

        # add axis/ground to the scene graph
        self.axis3D = loader.loadModel('misc/xyzAxis')
        self.axis3D.setPos( 0, 0, 0)
        self.axis3D.setScale( 0.1, 0.1, 0.1 )      # axis length=1 unit, so scale=0.1 since the 3Daxis mesh is 10 units
        self.axis3D.reparentTo(render)

        self.makeGround()

        # create usefull geom/NodePath but do not show them
        self.sphere = self.makeSphere(6) #loader.loadModel('sphere')
        self.cubeRBG = loader.loadModel('misc/rgbCube')
        self.axis_line = self.makeAxisLine()
        self.cylinder = self.makeCylinderX(7)            

        # Light: render.setLig
        self.light = render.attachNewNode("light")
        # self.dlight = NodePath( DirectionalLight('Directional_Light') )
        # self.dlight.reparentTo( self.light)
        plight_data = PointLight('plight')
        plight_data.setColor((0.9, 0.9, 0.9, 1))                                # LIGHT COLOR
        self.plight = render.attachNewNode(plight_data)
        self.plight.setPos(5, -25, 25)
        render.setLight(self.plight)
        self.sphere.instanceTo(self.plight)
        self.light.setColor( 0.9, 0.8, 0.2)                                     # SPHERE COLOR OF THE LIGHT

        # # light under the ground (user can not move it)
        # self.light_under = render.attachNewNode("light")
        # # self.dlight = NodePath( DirectionalLight('Directional_Light') )
        # # self.dlight.reparentTo( self.light)
        # plight_data = PointLight('plight_under')
        # plight_data.setColor((0.9, 0.9, 0.9, 1))                                # LIGHT COLOR
        # self.plight_under = render.attachNewNode(plight_data)
        # self.plight_under.setPos(0, 0, -20)
        # render.setLight(self.plight)
        # self.sphere.instanceTo(self.plight)
        # self.light.setColor( 0.9, 0.9, 0.9)                                     # SPHERE COLOR OF THE LIGHT
        alight = AmbientLight('alight')
        alight.setColor((0.3, 0.3, 0.3, 1))
        self.light_ambient = render.attachNewNode(alight)
        render.setLight(self.light_ambient)

        # true to display basic geometry in a test scene
        if False:
            ax = render.attachNewNode("test cylindre")
            self.cylinder.reparentTo(ax)
            self.placeCylinder( ax, LVector3(1,1,1), LVector3(3,3,3), 0.2)

            ax = render.attachNewNode("test sphere")
            self.sphere.instanceTo( ax )
            ax.setPos( 5, 5, 5)

            ax = render.attachNewNode("test axis")
            self.axis3D.instanceTo(ax)
            ax.setPos( -5, -5, 5)

            ax = render.attachNewNode("test axis")
            self.cubeRBG.instanceTo(ax)
            ax.setPos( -5, 5, 5)


        ### CAMERA stuff
        # dummy node for camera, we will rotate the dummy node fro camera rotation
        self.parentnode = render.attachNewNode('camera_parent')
        #self.parentnode.reparentTo(self.sphere) # inherit transforms
        self.parentnode.setEffect(CompassEffect.make(render)) # NOT inherit rotation

        # the camera
        base.cam.reparentTo(self.parentnode)
        base.cam.setPos( 10, -10, 10) # camera distance from model
        base.cam.setPos( 0, -10, 0) # camera distance from model
        base.cam.lookAt(self.parentnode)
        base.cam.setPos( 0, -10, 2) # camera distance from model
        # vars for camera rotation
        self.mousePos = (-1,-1)
        self.camOrient = (0,0)
        #base.cam.lookAt

        # camera control
        base.accept('wheel_up', lambda : base.cam.setY(base.cam.getY()+200 * globalClock.getDt()))
        base.accept('wheel_down', lambda : base.cam.setY(base.cam.getY()-200 * globalClock.getDt()))
        base.accept('arrow_left', lambda : base.cam.setX(base.cam.getX()-20 * globalClock.getDt()))
        base.accept('arrow_right', lambda : base.cam.setX(base.cam.getX()+20 * globalClock.getDt()))
        base.accept('page_up', lambda : base.cam.setZ(base.cam.getZ()+20 * globalClock.getDt()))
        base.accept('page_down', lambda : base.cam.setZ(base.cam.getZ()-20 * globalClock.getDt()))
        base.accept('control-arrow_up', lambda : base.plight.setY(base.plight.getY()+20 * globalClock.getDt()))
        base.accept('control-arrow_down', lambda : base.plight.setY(base.plight.getY()-20 * globalClock.getDt()))
        base.accept('control-arrow_left', lambda : base.plight.setX(base.plight.getX()-20 * globalClock.getDt()))
        base.accept('control-arrow_right', lambda : base.plight.setX(base.plight.getX()+20 * globalClock.getDt()))
        base.accept('control-page_up', lambda : base.plight.setZ(base.plight.getZ()+20 * globalClock.getDt()))
        base.accept('control-page_down', lambda : base.plight.setZ(base.plight.getZ()-20 * globalClock.getDt()))

        # base.accept('control-p', lambda : print("ctrl+p"))

        base.accept('control-arrow_up-repeat', lambda : messenger.send('control-arrow_up'))
        base.accept('control-arrow_down-repeat', lambda : messenger.send('control-arrow_down'))
        base.accept('control-arrow_right-repeat', lambda : messenger.send('control-arrow_right'))
        base.accept('control-arrow_left-repeat', lambda : messenger.send('control-arrow_left'))
        base.accept('control-page_up-repeat', lambda : messenger.send('control-page_up'))
        base.accept('control-page_down-repeat', lambda : messenger.send('control-page_down'))

        base.accept('arrow_up', lambda : messenger.send('wheel_up'))
        base.accept('arrow_down', lambda : messenger.send('wheel_down'))
        base.accept('page_up-repeat', lambda : messenger.send('page_up'))
        base.accept('page_down-repeat', lambda : messenger.send('page_down'))
        base.accept('arrow_left-repeat', lambda : messenger.send('arrow_left'))
        base.accept('arrow_right-repeat', lambda : messenger.send('arrow_right'))
        base.accept('arrow_up-repeat', lambda : messenger.send('arrow_up'))
        base.accept('arrow_down-repeat', lambda : messenger.send('arrow_down'))
        # base.accept('wheel_up' or 'arrow_up', lambda : base.cam.setY(base.cam.getY()+200 * globalClock.getDt()))
        # base.accept('wheel_down', lambda : base.cam.setY(base.cam.getY()-200 * globalClock.getDt()))

        # Add the spinCameraTask procedure to the task manager.
        self.accept("mouse1", self.mouseClick, [0, True])
        self.accept("mouse1-up", self.mouseClick, [0, False])
        inputState.watchWithModifiers('mouse1', 'mouse1')
        inputState.watchWithModifiers('mouse2', 'mouse2')
        self.task_camera = self.taskMgr.add( self.cameraMovementMouse, "cameraMovementMouse")
        self.animationStart()   # to create the variable
        self.animationStop()
        #self.taskMgr.stop()

        # list of key
        self.addKey("q", "Quit")
        self.addKey("escape", "Quit")
        self.addKey("a", "Toggle drawing of axis")
        self.addKey("g", "Toggle drawing of the ground")
        self.addKey("w", "Toggle wireframe")
        self.addKey("f", "Toggle frame rate")
        self.addKey("h", "Print this help")
        self.addKey("z", "Animation: run/stop")
        self.addKey("n", "Animation: next frame")
        self.addKey("b", "Animation: back one frame")
        self.addKey("control-i", "Interactive Python interpreter")
        self.addKey("control-d", "Interactive Python debug")
        self.addKey("control-l", "reLoad the code")


    def setTitle(self, title):
        WinProps = base.getProperties()
        WinProps.setTitle(title)
        base.requestProperties(WinProps)


    def addKey(self, k, helptxt=""):
        base.accept(k, self.keyHandler, [k])                    # panda: add event
        base.accept(k+"-repeat", self.keyHandler, [k])          # panda: add event, key is repeated if stay pressed
        if not hasattr(self, 'keyhelp'):
            self.keyhelp = {}
        if k in self.keyhelp:
            print("WARNING: PandaViewer::addKey: key "+k+" already exists. It will be replaced !")
        self.keyhelp[k] = helptxt


    def mouseClick(self, button, down):
        if down and button==0:
            md = base.win.getPointer(0)
            self.mousePos = (md.getX(), md.getY())
        if not down and button==0:
            self.mousePos = (-1, -1)


    # camera rotation task according to mouse movements
    def cameraMovementMouse(self, task):
        xold, yold = self.mousePos      
        #if inputState.isSet('mouse1'):
        if xold>=0 and yold>=0:
            md = base.win.getPointer(0)
            x = md.getX()
            y = md.getY()
            self.camOrient = ( self.camOrient[0] - (x - xold) * 0.5,  self.camOrient[1] - (y - yold) * 0.5 )
            self.mousePos = (x,y)
            self.parentnode.setHpr( self.camOrient[0],self.camOrient[1] ,0)
        return task.cont


    def makeAxisLine(self):
        length = 10
        lines = LineSegs()
        #x
        lines.setColor(1.0, 0.0, 0.0, 1.0)
        lines.moveTo(0, 0, 0)
        lines.drawTo(length, 0, 0)
        #y
        lines.setColor(0.0, 1.0, 0.0, 1.0)
        lines.moveTo(0, 0, 0)
        lines.drawTo(0, length, 0)
        #z
        lines.setColor(0.0, 0.0, 1.0, 1.0)
        lines.moveTo(0, 0, 0)
        lines.drawTo(0, 0, length)
        lines.setThickness(1)
        node = lines.create()
        return NodePath(node)


    def makeSphere(self, n):
        format = GeomVertexFormat.getV3n3cpt2()
        vdata = GeomVertexData('sphere', format, Geom.UHDynamic)

        vertex = GeomVertexWriter(vdata, 'vertex')
        normal = GeomVertexWriter(vdata, 'normal')
        color = GeomVertexWriter(vdata, 'color')
        texcoord = GeomVertexWriter(vdata, 'texcoord')
        tris = GeomTriangles(Geom.UHDynamic)

        idp = 0
        for j in range(n+1):
            a = -0.5*math.pi + float(j) * math.pi / n      #3.14159265 / n
            a_next = -0.5*math.pi + float(j+1) * math.pi / n      #3.14159265 / n
            for i in range(2*n+1):
                b = float(i) * 2.0*math.pi / (2.0*n)
                vertex.addData3(  cos(a)*cos(b),    cos(a)*sin(b), sin(a) )
                vertex.addData3(  cos(a_next)*cos(b),    cos(a_next)*sin(b), sin(a_next) )
                idp += 2
                normal.addData3(  cos(a)*cos(b),    cos(a)*sin(b), sin(a) )
                normal.addData3(  cos(a_next)*cos(b),    cos(a_next)*sin(b), sin(a_next) )
                texcoord.addData2f( float(i)/(2*n), float(j)/n)
                texcoord.addData2f( float(i)/(2*n), float(j+1)/n)
                if i>0:
                    tris.addVertices( idp-1, idp-3, idp-2 )
                    tris.addVertices( idp-2, idp-3, idp-4 )
                    # print("tri " + str(idp) + " "+ str(idp-1) + " "+ str(idp-2))
                    # print("tri " + str(idp-1) + " "+ str(idp-3) + " "+ str(idp-2))
        sph = Geom(vdata)
        sph.addPrimitive(tris)
        geom = GeomNode('cylinder')
        geom.addGeom(sph)
        node = NodePath(geom)
        return node


    def makeCylinderX(self, n):
        format = GeomVertexFormat.getV3n3cpt2()
        vdata = GeomVertexData('square', format, Geom.UHDynamic)

        vertex = GeomVertexWriter(vdata, 'vertex')
        normal = GeomVertexWriter(vdata, 'normal')
        color = GeomVertexWriter(vdata, 'color')
        texcoord = GeomVertexWriter(vdata, 'texcoord')
        tris = GeomTriangles(Geom.UHDynamic)

        for i in range(n+1):
            a = float(i) * 2.0*math.pi / n          #3.14159265 / n
            vertex.addData3( 0.0, cos(a), sin(a))
            vertex.addData3( 1.0, cos(a), sin(a))
            normal.addData3(normalized(0.0, cos(a), sin(a)))
            normal.addData3(normalized(0.0, cos(a), sin(a)))
            texcoord.addData2f( float(i)/n, 0.0)
            texcoord.addData2f( float(i)/n, 1.0)
            if i>0:
                # 2*i-2 ; 2*i-1 ; 2*i ; 2*i+1
                tris.addVertices( 2*i, 2*i-1, 2*i-2 )
                tris.addVertices( 2*i, 2*i+1, 2*i-1 )

        cyl = Geom(vdata)
        cyl.addPrimitive(tris)
        geom = GeomNode('cylinder')
        geom.addGeom(cyl)
        node = NodePath(geom)
        return node


    def makeSquareXY(self, n):
        """ make a square of n x n vertices on the plane XY. The plane is between 0 and 1 in X and in Y """
        format = GeomVertexFormat.getV3n3cpt2()
        vdata = GeomVertexData('square', format, Geom.UHDynamic)

        vertex = GeomVertexWriter(vdata, 'vertex')
        normal = GeomVertexWriter(vdata, 'normal')
        color = GeomVertexWriter(vdata, 'color')
        texcoord = GeomVertexWriter(vdata, 'texcoord')
        tris = GeomTriangles(Geom.UHDynamic)

        for i in range(n):
            for j in range(n):
                vertex.addData3( float(i)/(n-1), float(j)/(n-1), 0.0)
                normal.addData3( 0.0, 0.0, 1.0)
                texcoord.addData2f( float(i)/(n-1), float(j)/(n-1))
                if i>0 and j>0:
                    p0 = i*n + j
                    p1 = (i-1)*n + j
                    p2 = (i-1)*n + j-1
                    p3 = i*n + j-1
                    tris.addVertices( p0, p1, p2 )
                    tris.addVertices( p0, p2, p3 )

        cyl = Geom(vdata)
        cyl.addPrimitive(tris)
        geom = GeomNode('square')
        geom.addGeom(cyl)
        node = NodePath(geom)
        return node


    def placeCylinder(self, cyl, a, b, radius=-1):
        cyl.setPos( a )
        ab = b-a
        l = ab.length()
        if radius<0:
            radius = 0.1 * l
        cyl.setScale( l, radius, radius )
        cyl.lookAt( b )
        q = LQuaternion()
        q.setFromAxisAngle(  90.0, (0,0,1) )
        cyl.setQuat( q * cyl.getQuat() )


    def makeGround(self):
        """ make a ground with a grid. Size is 100x100 on XY plane """
        self.ground = render.attachNewNode("ground")

        self.ground_plane = self.makeSquareXY(40)           # mesh of 40x40, for lighting, since lighting is computer per vertex
        self.ground_plane.setPos(0,0,-0.01)                 # a little bit under 0
        self.ground_plane.setScale( 100, 100, 1 )
        self.ground_plane.setColor( 0.23, 0.45, 0.65)
        self.ground_plane.reparentTo( self.ground )

        self.grid_fine  = self.makeGridXY(100, 100, 1, 1,      1,      0.4, 0.4, 0.4 )
        self.grid_large = self.makeGridXY(100, 100, 10, 10,    3,      0.3, 0.3, 0.4)
        self.grid_large.reparentTo(self.ground)
        self.grid_fine.reparentTo(self.ground)

        self.ground.setPos(-50, -50, 0)                    # since the makePlane is 0..1; then scale 100, 100; and then translate -50, -50


    def makeGridXY(self, xmax, ymax, xstep, ystep, th, r=1.0, g=1.0, b=1.0, eps = 0.0):
        lines = LineSegs()
        lines.setColor(r, g, b, 1.0)
        lines.setThickness(th)
        x=0
        y=0
        while x <= xmax:
            lines.moveTo(x, 0, eps)
            lines.drawTo(x, ymax, eps)
            x += xstep
        while y <= ymax:
            lines.moveTo(0, y, eps)
            lines.drawTo(xmax, y, eps)
            y += ystep
        node = lines.create()
        return NodePath(node)


    # key handler
    def keyHandler(self,key):
        if key=="q" or key=="escape" :
            print("quit")
            #sys.exit(0)
            self.stop()
        elif key=='a':
            if not self.axis3D.has_parent():
                self.axis3D.reparentTo(render)
            else:
                self.axis3D.detachNode()
        elif key=='g':
            if not self.ground.has_parent():
                self.ground.reparentTo(render)
            else:
                self.ground.detachNode()
        elif key=='w':
            if render.hasRenderMode():
                render.clearRenderMode()
                # render.setRenderModeFilled()
            else:
                render.setRenderModeWireframe()
        elif key=='h':
            self.help()
        elif key=='n':
            self.animate()
        elif key=='f':
            if base.frameRateMeter is None:
                base.setFrameRateMeter(True)
                print("FrameRate: on")
            else:
                base.setFrameRateMeter(False)
                print("FrameRate: off")
        elif key=="z":
            if self.isAnimationOn():
                self.animationStop()
            else:
                self.animationStart()
        elif key=="control-i":
            # from imp import reload
            # code.interact(local=locals())
            #print("rerun")
            # IPython.start_ipython(argv=[])
            #import IPython
            #IPython.start_ipython(argv=[])
            #IPython.embed()
            #global interactive_debug
            self.interactive_debug=True
            print("stop for debug")
            self.stop()
        elif key=="control-l":
            #from IPython.lib.deepreload import reload as dreload
            #mod = sys.modules[__file__]
            #dreload(mod)
            print("change 19")
        elif key=="control-b":
            breakpoint()
        else:
            print("key unmanaged: "+key)


    def isAnimationOn(self):
        return self.animation_task.is_alive()

    def animationStart(self):
        self.animation_task = self.taskMgr.add( self.animateTask, "animate")
        print("anim start")

    def animationStop(self):
        self.taskMgr.remove( self.animation_task )
        print("anim stop")

    def stop(self):
        # Set a flag so we will stop before beginning next frame
        #self.running = 0
        self.taskMgr.stop()

    def help(self):
        print("Help")
        print("Animation On/Off: "+str(self.isAnimationOn()))
        for key in self.keyhelp:
            print(key + ": " + self.keyhelp[key])

    def animate(self):
        print("animate")

    def animateTask(self, task):
        self.animate()
        return task.cont



if __name__ == "__main__":
    app = PandaViewer()
    app.run()
    #global interactive_debug
    if app.interactive_debug==True:
        app.interactive_debug = False
        from IPython.lib.deepreload import reload as dreload
        import IPython
        import PandaViewer
        IPython.embed()
        #interactive_debug
        #interactive_debug=False


    print("ending...")
